# This file is part of the FreeType project.
#
# It builds the library and test programs for Microsoft Visual C++ 1.x
# and Microsoft C/C++ v.7.0 16 bits compilers for MS-DOS, large model.
#
# You will need NMAKE.
#
#
# Use this file while in the 'test' directory with the following statement:
#
#   make -farch/msdos/Makefile.VC
#
#
# A debug version can be obtained with
#
#   make -DDEBUG -farch/msdos/Makefile.VC

ARCH = arch\msdos
FT_MAKEFILE = $(ARCH)\Makefile.VC

CC = cl /nologo

LIBDIR = ..\lib
INCDIRS = -I$(LIBDIR) -I$(LIBDIR)\$(ARCH) -I. -I$(LIBDIR)\extend

!ifndef DEBUG
CFLAGS = -AL -Za -Ox -Gr -G3 -Gy -H40 $(INCDIRS) -DNO_DISPLAY -DNO_EXTENSION
LDFLAGS = -AL
!else
CFLAGS = -Zi -Ge -AL -Za -Gr -H40 $(INCDIRS)
LDFLAGS = -Zi -AL
!endif


# full-screen MSDOS driver
GDRIVER = $(ARCH)\gfs_dos.c

GSRC = display.c gmain.c blitter.c $(GDRIVER)

GOBJ = $(GSRC:.c=.obj)


SRC = arabic.c \
      common.c \
      ftdump.c \
      fterror.c \
      ftlint.c \
      ftmetric.c \
      ftsbit.c \
      ftstring.c \
      ftstrpnm.c \
      ftstrtto.c \
      fttimer.c \
      ftview.c \
      ftzoom.c

OBJ = $(SRC:.c=.obj)


.c.obj:
	$(CC) @<<
              $(CFLAGS) -c -Fo$* $<
<<KEEP

EXEFILES = ftdump.exe \
           fterror.exe \
           ftlint.exe \
           ftmetric.exe \
           ftsbit.exe \
           ftstring.exe \
           ftstrpnm.exe \
           ftstrtto.exe \
           fttimer.exe \
           ftview.exe \
           ftzoom.exe

!ifndef DEBUG
# Skiped if DEBUG build
all: freetype $(EXEFILES)

!endif

debug: freetype_debug $(EXEFILES)

freetype:
	cd $(LIBDIR)
	$(MAKE) /f$(FT_MAKEFILE) all
	cd ..\test

freetype_debug:
	cd $(LIBDIR)
	$(MAKE) /f$(FT_MAKEFILE) DEBUG=1 debug
	cd ..\test

.obj.exe:
	$(CC) @<<
              $(LDFLAGS) /Fe$* $**
<<

ftzoom.exe: $(GOBJ) ftzoom.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) /Fe$* $**

ftview.exe: $(GOBJ) ftview.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) ftview.obj $(GOBJ) common.obj $(LIBDIR)\libttf.lib

ftstring.exe: $(GOBJ) ftstring.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) ftstring.obj $(GOBJ) common.obj $(LIBDIR)\libttf.lib

ftstrtto.exe: $(GOBJ) ftstrtto.obj common.obj arabic.obj $(LIBDIR)\libttf.lib

fttimer.exe: $(GOBJ) fttimer.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) fttimer.obj $(G1OBJ) common.obj $(LIBDIR)\libttf.lib

ftlint.exe: ftlint.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) -e$* ftlint.obj common.obj $(LIBDIR)\libttf.lib

ftdump.exe: ftdump.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) -e$* ftdump.obj common.obj $(LIBDIR)\libttf.lib

ftstrpnm.exe: ftstrpnm.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) -e$* ftstrpnm.obj common.obj $(LIBDIR)\libttf.lib

ftsbit.exe: ftsbit.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) -e$* ftsbit.obj common.obj $(LIBDIR)\libttf.lib

ftmetric.exe: ftmetric.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) -e$* ftmetric.obj common.obj $(LIBDIR)\libttf.lib

fterror.exe: fterror.obj common.obj $(LIBDIR)\libttf.lib
#	$(CC) $(LDFLAGS) -e$* fterror.obj common.obj $(LIBDIR)\libttf.lib


clean: do_clean
	cd $(LIBDIR)
	$(MAKE) -f$(FT_MAKEFILE) clean
	cd ..\test

distclean: do_clean
	cd $(LIBDIR)
	$(MAKE) -f$(FT_MAKEFILE) distclean
	cd ..\test
	-del *.exe

do_clean:
	-del *.obj
	-del arch\msdos\*.obj


!include "$(ARCH)\depend.dos"

# end of Makefile.VC
